// dspelim.cpp : Defines the entry point for the DLL application.
//

#include <windows.h>
#include <commctrl.h>
#include "dsp.h"
#include "resource.h"

// this declaration suppresses linking an unneeded 8K module
extern "C" {
BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	return TRUE;
}
}

// pan value
int panvalue=0;

// pan control window
HWND pan_hwnd;

// module getter.
winampDSPModule* getModule(int which);

static BOOL CALLBACK panProc(HWND hwndDlg, UINT uMsg, WPARAM wParam,LPARAM lParam);

void config(struct winampDSPModule* tmod);
int initnone(struct winampDSPModule* tmod);
int initfl(struct winampDSPModule* tmod);
int initml(struct winampDSPModule* tmod);
int initcn(struct winampDSPModule* tmod);
int initmr(struct winampDSPModule* tmod);
int initfr(struct winampDSPModule* tmod);
void quit(struct winampDSPModule* tmod);
int remove_channel(struct winampDSPModule* tmod, short int* samples, int numsamples, int bps, int nch, int srate);

// Module header, includes version, description, and address of the module retriever function
winampDSPHeader hdr = { DSP_HDRVER, "Channel Eliminator", getModule };

static int* pan = 0;
static int panfl = -50;
static int panml = -30;
static int pancn =   0;
static int panmr =  30;
static int panfr =  50;

winampDSPModule modnone =
{
	"(None)",
	0,	// hwndParent
	0,	// hDllInstance
	config,
	initnone,
	remove_channel,
	quit
};

// math.h exports global modfl
winampDSPModule modflx =
{
	"Remove Far Left Channel",
	0,	// hwndParent
	0,	// hDllInstance
	config,
	initfl,
	remove_channel,
	quit
};

winampDSPModule modml =
{
	"Remove Middle Left Channel",
	0,	// hwndParent
	0,	// hDllInstance
	config,
	initml,
	remove_channel,
	quit
};

winampDSPModule modcn =
{
	"Remove Center Channel",
	0,	// hwndParent
	0,	// hDllInstance
	config,
	initcn,
	remove_channel,
	quit
};

winampDSPModule modmr =
{
	"Remove Middle Right Channel",
	0,	// hwndParent
	0,	// hDllInstance
	config,
	initmr,
	remove_channel,
	quit
};

winampDSPModule modfr =
{
	"Remove Far Right Channel",
	0,	// hwndParent
	0,	// hDllInstance
	config,
	initfr,
	remove_channel,
	quit
};


extern "C" {
// this is the only exported symbol. returns our main header.
__declspec( dllexport ) winampDSPHeader* winampDSPGetHeader2()
{
	return &hdr;
}

}

// getmodule routine from the main header. Returns 0 if an invalid module was requested,
// otherwise returns either mod1 or mod2 depending on 'which'.
winampDSPModule* getModule(int which)
{
	switch (which)
	{
		case 0: return &modnone;
		case 1: return &modflx;
		case 2: return &modml;
		case 3: return &modcn;
		case 4: return &modmr;
		case 5: return &modfr;
		default:return 0;
	}
}

void config(struct winampDSPModule* tmod)
{
	MessageBox(tmod->hwndParent,"Freeware - Al Stevens",
									"Configuration",MB_OK);
}

int initnone(struct winampDSPModule* tmod)
{
	pan = 0;
	return 0;
}

void init(struct winampDSPModule *tmod)
{
	pan_hwnd=
		CreateDialog(tmod->hDllInstance,
					 MAKEINTRESOURCE(IDD_DIALOG1),
					 tmod->hwndParent,
					 panProc
					);
	SendMessage(pan_hwnd, WM_SETTEXT, 0, (LPARAM) tmod->description);
	ShowWindow(pan_hwnd,SW_SHOW);
}

int initfl(struct winampDSPModule* tmod)
{
	pan = &panfl;
	init(tmod);
	return 0;
}
int initml(struct winampDSPModule* tmod)
{
	pan = &panml;
	init(tmod);
	return 0;
}
int initcn(struct winampDSPModule* tmod)
{
	pan = &pancn;
	init(tmod);
	return 0;
}
int initmr(struct winampDSPModule* tmod)
{
	pan = &panmr;
	init(tmod);
	return 0;
}
int initfr(struct winampDSPModule* tmod)
{
	pan = &panfr;
	init(tmod);
	return 0;
}

void quit(struct winampDSPModule* tmod)
{
	if (pan_hwnd)	{
		DestroyWindow(pan_hwnd);
		pan_hwnd=0;
		pan = 0;
	}
}

int remove_channel(struct winampDSPModule* tmod, short int* samples, int numsamples, int bps, int nch, int srate)
{
	if (pan)	{
		int x = numsamples;
		while (x--)	{
			int lf = (*samples);
			int rt = *(samples+1);

			lf = (lf * (100 + *pan)) / 100;
			rt = (rt * (100 - *pan)) / 100;

			*samples++ = min(max((rt-lf), -32768), 32768);
			*samples++ = (min(max((lf-rt), -32768), 32768)) * -1;
		}
	}
	return numsamples;
}

static void setpantext(HWND hwndDlg)
{
	char str[20];
	wsprintf(str,"%s%d%%", *pan > 0 ? "+" : "", *pan);
	SetDlgItemText(hwndDlg, IDC_PANVALUE, str);
}

static BOOL CALLBACK panProc(HWND hwndDlg, UINT uMsg, WPARAM wParam,LPARAM lParam)
{
	if (uMsg == WM_INITDIALOG)	{
		SendDlgItemMessage(hwndDlg, IDC_SLIDER1, TBM_SETRANGEMAX, 0, 100);
		SendDlgItemMessage(hwndDlg, IDC_SLIDER1, TBM_SETRANGEMIN, 0, -100);
	    SendDlgItemMessage(hwndDlg, IDC_SLIDER1, TBM_SETPOS, 1, 1);
	    SendDlgItemMessage(hwndDlg, IDC_SLIDER1, TBM_SETPOS, 1, *pan);
		setpantext(hwndDlg);
	}
	if (uMsg == WM_HSCROLL)	{
		HWND swnd = (HWND) lParam;
		if (swnd == GetDlgItem(hwndDlg,IDC_SLIDER1))	{
			*pan = SendDlgItemMessage(hwndDlg,IDC_SLIDER1,TBM_GETPOS,0,0);
			setpantext(hwndDlg);
		}
	}
	return 0;
}

