function fixed_pt_iteration_1
%
% FIXED_PT_ITERATION_1   un-optimized fixed-point iteration
%

num_it = 100; % # of iterations
num_vars = 2;
sigma_term = .125;

load variables.mat; % get F, x & y axes

% get starting point for fixed-pt iteration
fixed_pt_solution = get_initial_guess(x_axis, y_axis, F);

length_x_axis = length(x_axis);
length_y_axis = length(y_axis);

for n=1:num_it
    
    denominator = 0; % denominator of expression given by equations 1(a) & 1(b) 
    numerator = [0 0]; % numerator of expression given by equations 1(a) & 1(b)
    
    for ii = 1:length(x_axis)
        for jj = 1:length(y_axis)
            
            xy = fixed_pt_solution; % [ xi yi ]
            axis_pts = [x_axis(ii) y_axis(jj)];
            term = F(ii,jj)*gaussian(xy, axis_pts, sigma_term);
            denominator = denominator + term;
            numerator(1) = numerator(1) + x_axis(ii) * term;
            numerator(2) = numerator(2) + y_axis(jj) * term;
                 
        end
    end

    % update refined answer
    fixed_pt_solution = numerator./denominator;    
    
end

disp(sprintf('Answer is (%f,%f)', fixed_pt_solution(1), fixed_pt_solution(2)));

function initial_guess = get_initial_guess(x_axis, y_axis, F)
%
% GET_INITIAL_GUESS - returns starting point for fixed point iteration
%                     evaluates expression given by 2(a) & 2(b)
%

x_initial = 0;
y_initial = 0;
[M N] = size(F);

for ii=1:M
    for jj=1:N
        x_initial = x_initial + x_axis(ii) * F(ii,jj);
        y_initial = y_initial + y_axis(jj) * F(ii,jj);
    end
end

initial_guess = [ x_initial y_initial ] ./ sum(F(:));

function term = gaussian(xy, axis_pts, sigma_term)
%
% GAUSSIAN - evaluates 2-dimensional gaussian given by eq 3
%

term = 1.0;
for dim=1:2
    term = term * exp(-(xy(dim)-axis_pts(dim)) * (xy(dim)-axis_pts(dim)) / (sigma_term));
end